'Der Datentyp "pos" nimmt die Koordinaten eines Rechtecks
'in einem Bildfeld und das DeviceHandle des Bildfelds
'auf.

Type Pos
    X As Integer            'X-Position (linker Rand)
    Y As Integer            'Y-Position (oberer Rand)
    W As Integer            'Breite des Rechtecks
    H As Integer            'Hhe des Rechtecks
    HDC As Integer          'DeviceContext des Bildfelds
End Type

'Der Datentyp Sprite enthlt die Informationen, die fr einen
'Sprite wichtig sind: Die Position der Maske, die Position der
'Bilddaten und die Position des Sprites auf dem Ausgabe-Bildschirm.
'Letzteres wre eigentlich nicht unbedingt ntigt, da die X- und
'Y-Position immer direkt an SPRITE_DRAW bergeben werden. Jedoch
'Kann man eine bestimmte Ausgangsgre des Sprites festlegen, so
'da er vergrert oder verkleinert dargestellt wird

'Bit und Maske werden mit den SPRITE_SET... Funktionen gesetzt

Type Sprite
    Bit As Pos              'Bilddaten
    Maske As Pos            'Maske
    Nach As Pos             'Position, an der der Sprite
                            'angezeigt werden soll
End Type
      

'Nun einige Definitionen aus der Win-API:

Global Const BIT_COPY = &HCC0020
Global Const BIT_AND = &H8800C6
Global Const BIT_INVERT = &H660046
Global Const BIT_BLACK = &H42&

Declare Function BitBlt Lib "Gdi" (ByVal NachHdc%, ByVal X%, ByVal Y%, ByVal W%, ByVal H%, ByVal vonHdc%, ByVal VonX%, ByVal VonY%, ByVal Modus&) As Integer
Declare Function StretchBlt Lib "Gdi" (ByVal NachHdc%, ByVal X%, ByVal Y%, ByVal W%, ByVal H%, ByVal vonHdc%, ByVal VonX%, ByVal VonY%, ByVal vonW%, ByVal vonH%, ByVal Modus&) As Integer

'
' Diese Prozedur setzt den Inhalt der Position P, die bergeben
' wird. So kann man zum Beispiel die den Maske-Teil eines Sprites
' ndern:
'
' POS_SET Sprite1.Maske, 10, 10, 100, 100, (Bild1.Hdc)
'
' Diese Funktion wird normal nicht vom Programmierer aufgerufen,
' sondern von anderen Prozeduren.
'
Sub POS_SET (P As Pos, X, Y, W, H, PicBox As PictureBox)

    P.X = X
    P.Y = Y
    P.W = W
    P.H = H
    P.HDC = PicBox.hDC

End Sub

'
' Diese Prozedur setzt den Inhalt der Position P, die bergeben
' wird. So kann man zum Beispiel die den Maske-Teil eines Sprites
' ndern:
'
' POS_SET_MAX Sprite1.Maske, Bild1
'
' Diese Funktion setzt das Rechteck auf das ganze Bildfeld.
' Der Aufruf kommt also gleich mit:
'
' POS_SET P, 0, 0, Bild1.Width, Bild1.Height, (Bild1.HDC)
'
Sub POS_SET_MAX (P As Pos, PicBox As PictureBox)

    P.X = 0
    P.Y = 0
    P.W = PicBox.ScaleWidth
    P.H = PicBox.ScaleHeight
    P.HDC = PicBox.hDC

End Sub

'
' Sprite zeichenen.
' Der bergebene Sprite wird an der Position X,Y mit den
' ber den Sprite eingestellten Werte (Breite, Hhe etc)
' eingezeichnet
'
Sub SPRITE_DRAW (S As Sprite, X, Y, PicBox As PictureBox)

    S.Nach.HDC = PicBox.hDC
    r% = StretchBlt(S.Nach.HDC, X, Y, S.Nach.W, S.Nach.H, S.Maske.HDC, S.Maske.X, S.Maske.Y, S.Maske.W, S.Maske.H, BIT_AND)
    r% = StretchBlt(S.Nach.HDC, X, Y, S.Nach.W, S.Nach.H, S.Bit.HDC, S.Bit.X, S.Bit.Y, S.Bit.W, S.Bit.H, BIT_INVERT)
    
End Sub

'
' Sprite festlegen. Der Sprite wird auf die bergebenen Werte
' eingestellt. Diese werden in Form von Daten des Typs POS
' bergeben
'
Sub SPRITE_SET (S As Sprite, Bit As Pos, Maske As Pos)

    S.Bit = Bit
    S.Maske = Maske
    S.Nach.X = 0
    S.Nach.Y = 0
    S.Nach.W = Bit.W
    S.Nach.H = Bit.H
    
End Sub

Sub SPRITE_SET_MAX (S As Sprite, Bit As PictureBox, Maske As PictureBox)
    
    Dim B As Pos, M As Pos, N As Pos

    POS_SET_MAX B, Bit
    POS_SET_MAX M, Maske
    
    SPRITE_SET S, B, M

End Sub

'
' Erstellt mehrere Sprites als Datenfeld (Sprite(1), Sprite(2)...).
' Diese Funktion bekommt keine einzelnen Werte fr die Sprites
' bergeben sondern erstellt die Sprites an Hand eines Gitters.
'
' Es wird davon ausgegangen, da in den Ressourcen mit dem Handle
' Pb1 und Pb2 Bilder vorliegen, die in einzelne Kstchen aufgeteilt sind,
' wobei jedes Kstchen W-Pixel breit und H-Pixel hoch ist. Dadurch kann
' man sehr leicht eine Vielzahl von Sprites erstellen, ohne Einzelbilder
' zeichnen zu mssen.
'
'
'
Sub SPRITE_SET_MESH (Mesh() As Sprite, Start, X, Y, W, H, Pb1 As PictureBox, Pb2 As PictureBox)

    YPos = 1
    I = Start

    For Iy = 1 To Y
        XPos = 1
        For Ix = 1 To X
            SPRITE_SET_SYNC Mesh(I), XPos, YPos, W, H, Pb1, Pb2
            I = I + 1
            XPos = XPos + W + 1
        Next Ix
        YPos = YPos + H + 1
    Next Iy
    Start = I

End Sub

'
' Diese Prozedur setzt ebenfalls die Daten fr einen Sprite fest. Hierbei
' mssen jedoch die Positionen der Maske und der Bilddaten bereinstimmen
'
Sub SPRITE_SET_SYNC (S As Sprite, X, Y, W, H, Bit As PictureBox, Maske As PictureBox)

    S.Bit.X = X
    S.Bit.Y = Y
    S.Bit.W = W
    S.Bit.H = H
    S.Bit.HDC = Bit.hDC
    S.Maske = S.Bit
    S.Maske.HDC = Maske.hDC
    
    S.Nach.X = 0
    S.Nach.Y = 0
    S.Nach.W = W
    S.Nach.H = H
    
End Sub

