Attribute VB_Name = "MODUL_SPRITE"
Option Explicit

'Nun einige Definitionen aus der Win-API:
Public Declare Function BitBlt Lib "Gdi32" (ByVal _
       NachHdc As Long, ByVal X As Long, ByVal Y _
       As Long, ByVal W As Long, ByVal H As Long, _
       ByVal vonHdc As Long, ByVal VonX As Long, _
       ByVal VonY As Long, ByVal Modus As Long) _
       As Long
       
Public Declare Function StretchBlt Lib "Gdi32" (ByVal _
       NachHdc As Long, ByVal X As Long, ByVal Y As _
       Long, ByVal W As Long, ByVal H As Long, _
       ByVal vonHdc As Long, ByVal VonX As Long, _
       ByVal VonY As Long, ByVal vonW As Long, _
       ByVal vonH As Long, ByVal Modus As Long) _
       As Long

Public Const BIT_COPY = &HCC0020
Public Const BIT_AND = &H8800C6
Public Const BIT_INVERT = &H660046
Public Const BIT_BLACK = &H42&


'Der Datentyp "pos" nimmt die Koordinaten eines Rechtecks
'in einem Bildfeld und das DeviceHandle des Bildfelds
'auf.

Private Type Pos
  X As Long            'X-Position (linker Rand)
  Y As Long            'Y-Position (oberer Rand)
  W As Long            'Breite des Rechtecks
  H As Long            'Hhe des Rechtecks
  HDC As Long          'DeviceContext des Bildfelds
End Type

'Der Datentyp Sprite enthlt die Informationen, die fr einen
'Sprite wichtig sind: Die Position der Maske, die Position der
'Bilddaten und die Position des Sprites auf dem Ausgabe-Bildschirm.
'Letzteres wre eigentlich nicht unbedingt ntigt, da die X- und
'Y-Position immer direkt an SPRITE_DRAW bergeben werden. Jedoch
'Kann man eine bestimmte Ausgangsgre des Sprites festlegen, so
'da er vergrert oder verkleinert dargestellt wird

'Bit und Maske werden mit den SPRITE_SET... Funktionen gesetzt

Public Type Sprite
  Bit As Pos              'Bilddaten
  Maske As Pos            'Maske
  Nach As Pos             'Position, an der der Sprite
                          'angezeigt werden soll
End Type
      

' Diese Prozedur setzt den Inhalt der Position P, die bergeben
' wird. So kann man zum Beispiel die den Maske-Teil eines Sprites
' ndern:
'
' POS_SET Sprite1.Maske, 10, 10, 100, 100, (Bild1.Hdc)
'
' Diese Funktion wird normal nicht vom Programmierer aufgerufen,
' sondern von anderen Prozeduren.
Public Sub POS_SET(P As Pos, X As Long, Y As Long, _
                   W As Long, H As Long, PicBox As PictureBox)

    P.X = X
    P.Y = Y
    P.W = W
    P.H = H
    P.HDC = PicBox.HDC

End Sub

' Diese Prozedur setzt den Inhalt der Position P, die bergeben
' wird. So kann man zum Beispiel die den Maske-Teil eines Sprites
' ndern:
'
' POS_SET_MAX Sprite1.Maske, Bild1
'
' Diese Funktion setzt das Rechteck auf das ganze Bildfeld.
' Der Aufruf kommt also gleich mit:
'
' POS_SET P, 0, 0, Bild1.Width, Bild1.Height, (Bild1.HDC)
Public Sub POS_SET_MAX(P As Pos, PicBox As PictureBox)
  P.X = 0
  P.Y = 0
  P.W = PicBox.ScaleWidth
  P.H = PicBox.ScaleHeight
  P.HDC = PicBox.HDC
End Sub

' Sprite zeichenen.
' Der bergebene Sprite wird an der Position X,Y mit den
' ber den Sprite eingestellten Werte (Breite, Hhe etc)
' eingezeichnet
Public Sub SPRITE_DRAW(S As Sprite, X As Long, Y As Long, PicBox As PictureBox)
  S.Nach.HDC = PicBox.HDC
  Call StretchBlt(S.Nach.HDC, X, Y, S.Nach.W, S.Nach.H, S.Maske.HDC, S.Maske.X, S.Maske.Y, S.Maske.W, S.Maske.H, BIT_AND)
  Call StretchBlt(S.Nach.HDC, X, Y, S.Nach.W, S.Nach.H, S.Bit.HDC, S.Bit.X, S.Bit.Y, S.Bit.W, S.Bit.H, BIT_INVERT)
End Sub

' Sprite festlegen. Der Sprite wird auf die bergebenen Werte
' eingestellt. Diese werden in Form von Daten des Typs POS
' bergeben
Public Sub SPRITE_SET(S As Sprite, Bit As Pos, Maske As Pos)
  S.Bit = Bit
  S.Maske = Maske
  S.Nach.X = 0
  S.Nach.Y = 0
  S.Nach.W = Bit.W
  S.Nach.H = Bit.H
End Sub

Public Sub SPRITE_SET_MAX(S As Sprite, Bit As PictureBox, Maske As PictureBox)
  Dim B As Pos, M As Pos, N As Pos

    POS_SET_MAX B, Bit
    POS_SET_MAX M, Maske
    
    SPRITE_SET S, B, M
End Sub

' Erstellt mehrere Sprites als Datenfeld (Sprite(1), Sprite(2)...).
' Diese Funktion bekommt keine einzelnen Werte fr die Sprites
' bergeben sondern erstellt die Sprites an Hand eines Gitters.
'
' Es wird davon ausgegangen, da in den Ressourcen mit dem Handle
' Pb1 und Pb2 Bilder vorliegen, die in einzelne Kstchen aufgeteilt sind,
' wobei jedes Kstchen W-Pixel breit und H-Pixel hoch ist. Dadurch kann
' man sehr leicht eine Vielzahl von Sprites erstellen, ohne Einzelbilder
' zeichnen zu mssen.
Public Sub SPRITE_SET_MESH(Mesh() As Sprite, Start, X As Long, Y As Long, W As Long, H As Long, Pb1 As PictureBox, Pb2 As PictureBox)
  Dim Xpos As Long, Ypos As Long, i As Long, ix As Long, iy As Long
  
    Ypos = 1
    i = Start

    For iy = 1 To Y
        Xpos = 1
        For ix = 1 To X
            SPRITE_SET_SYNC Mesh(i), Xpos, Ypos, W, H, Pb1, Pb2
            i = i + 1
            Xpos = Xpos + W + 1
        Next ix
        Ypos = Ypos + H + 1
    Next iy
    Start = i

End Sub

' Diese Prozedur setzt ebenfalls die Daten fr einen Sprite fest. Hierbei
' mssen jedoch die Positionen der Maske und der Bilddaten bereinstimmen
Public Sub SPRITE_SET_SYNC(S As Sprite, X, Y, W, H, _
                           Bit As PictureBox, Maske As PictureBox)
  S.Bit.X = X
  S.Bit.Y = Y
  S.Bit.W = W
  S.Bit.H = H
  S.Bit.HDC = Bit.HDC
  S.Maske = S.Bit
  S.Maske.HDC = Maske.HDC
    
  S.Nach.X = 0
  S.Nach.Y = 0
  S.Nach.W = W
  S.Nach.H = H
End Sub

